@@ -10,8 +10,9 @@ no warnings 'uninitialized';
 
 use Yote::Cron;
 use Yote::Login;
+
+use Email::Valid;
 use Mail::Sender;
-use MIME::Lite;
 
 use base 'Yote::AppRoot';
 
@@ -284,7 +285,6 @@ sub recovery_reset_password {
 sub remove_login {
     my( $self, $args, $acct, $env ) = @_;
     my $login = $acct->get_login();
-
     if( $login && 
         Yote::ObjProvider::encrypt_pass($args->{p}, $login->get_handle()) eq $login->get__password() &&
         $args->{h} eq $login->get_handle() &&
